@@ -14,6 +14,14 @@
 
 //! OpenDAL is the Open Data Access Layer that connect the whole world together.
 //!
+//! # Supported Services
+//!
+//! | Services | Description |
+//! | -------- | ----------- |
+//! | [fs][crate::services::fs] | POSIX alike file system. |
+//! | [memory][crate::services::memory] | In memory backend support. |
+//! | [s3][crate::services::s3] | AWS S3 alike services. |
+//!
 //! # Example
 //!
 //! ```
@@ -31,26 +39,19 @@
 //!     let o = op.object("test_file");
 //!
 //!     // Write data info file;
-//!     let w = o.writer();
-//!     let n = w
-//!        .write_bytes("Hello, World!".to_string().into_bytes())
-//!         .await?;
+//!     let _ = o.write("Hello, World!").await?;
 //!
 //!     // Read data from file;
-//!     let mut r = o.reader();
-//!     let mut buf = vec![];
-//!     let n = r.read_to_end(&mut buf).await?;
+//!     let bs = o.read().await?;
 //!
 //!     // Read range from file;
-//!     let mut r = o.range_reader(10, 1);
-//!     let mut buf = vec![];
-//!     let n = r.read_to_end(&mut buf).await?;
+//!     let bs = o.range_read(1..=11).await?;
 //!
 //!     // Get file's Metadata
 //!    let meta = o.metadata().await?;
 //!
-//!     // List current dir.
-//!     let mut obs = op.objects("").map(|o| o.expect("list object"));
+//!     // List dir.
+//!     let mut obs = op.objects("").await?.map(|o| o.expect("list object"));
 //!     while let Some(o) = obs.next().await {
 //!         let meta = o.metadata().await?;
 //!         let path = meta.path();
@@ -64,21 +65,18 @@
 //!     Ok(())
 //! }
 //! ```
-//!
-//! ## Supported Services
-//!
-//! - [fs][crate::services::fs]: POSIX alike file system.
-//! - [memory][crate::services::memory]: In memory backend support.
-//! - [s3][crate::services::s3]: AWS services like S3.
-extern crate core;
 
+// Private module with public types, they will be accessed via `opendal::Xxxx`
 mod accessor;
 pub use accessor::Accessor;
 
 mod io;
-pub use io::BoxedAsyncReader;
-pub use io::Reader;
-pub use io::Writer;
+pub use io::BytesRead;
+pub use io::BytesReader;
+pub use io::BytesSink;
+pub use io::BytesStream;
+pub use io::BytesWrite;
+pub use io::BytesWriter;
 
 mod layer;
 pub use layer::Layer;
@@ -87,21 +85,24 @@ mod operator;
 pub use operator::Operator;
 
 mod object;
-pub use object::BoxedObjectStream;
 pub use object::Metadata;
 pub use object::Object;
 pub use object::ObjectMode;
 pub use object::ObjectStream;
+pub use object::ObjectStreamer;
 
 mod scheme;
 pub use scheme::Scheme;
 
-pub mod credential;
-pub mod error;
-pub mod readers;
-
+// Public modules, they will be accessed via `opendal::io_util::Xxxx`
+pub mod io_util;
 pub mod ops;
 pub mod services;
 
-#[cfg(test)]
-pub mod tests;
+// Private modules, internal use only.
+//
+// Please don't export any type from this module.
+mod error;
+
+#[deprecated]
+pub mod readers;
