@@ -14,11 +14,15 @@
 
 use std::cmp::min;
 use std::collections::HashMap;
-use std::pin::Pin;
+use std::fmt::Debug;
+use std::fmt::Formatter;
+use std::io;
+use std::io::Error;
+use std::io::ErrorKind;
+use std::io::Result;
+use std::mem;
 use std::str::FromStr;
 use std::sync::Arc;
-use std::task::Context;
-use std::task::Poll;
 
 use anyhow::anyhow;
 use async_trait::async_trait;
@@ -32,64 +36,97 @@ use hyper::Body;
 use log::debug;
 use log::error;
 use log::info;
-use log::warn;
 use metrics::increment_counter;
 use minitrace::trace;
 use reqsign::services::azure::storage::Signer;
 use time::format_description::well_known::Rfc2822;
 use time::OffsetDateTime;
 
-use crate::credential::Credential;
-use crate::error::Error;
-use crate::error::Kind;
-use crate::error::Result;
+use crate::error::other;
+use crate::error::BackendError;
+use crate::error::ObjectError;
+use crate::io_util::new_http_channel;
+use crate::io_util::HttpBodyWriter;
 use crate::object::Metadata;
-use crate::ops::HeaderRange;
+use crate::ops::BytesRange;
 use crate::ops::OpDelete;
+use crate::ops::OpList;
 use crate::ops::OpRead;
 use crate::ops::OpStat;
 use crate::ops::OpWrite;
-use crate::readers::ReaderStream;
+use crate::services::azblob::object_stream::AzblobObjectStream;
 use crate::Accessor;
-use crate::BoxedAsyncReader;
+use crate::BytesReader;
+use crate::BytesWriter;
 use crate::ObjectMode;
+use crate::ObjectStreamer;
 
-pub const DELETE_SNAPSHOTS: &str = "x-ms-delete-snapshots";
-pub const BLOB_TYPE: &str = "x-ms-blob-type";
+pub const X_MS_BLOB_TYPE: &str = "x-ms-blob-type";
 
-#[derive(Default, Debug, Clone)]
+#[derive(Default, Clone)]
 pub struct Builder {
     root: Option<String>,
     container: String,
-    credential: Option<Credential>,
     endpoint: Option<String>,
+    account_name: Option<String>,
+    account_key: Option<String>,
+}
+
+impl Debug for Builder {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        let mut ds = f.debug_struct("Builder");
+
+        ds.field("root", &self.root);
+        ds.field("container", &self.container);
+        ds.field("endpoint", &self.endpoint);
+
+        if self.account_name.is_some() {
+            ds.field("account_name", &"<redacted>");
+        }
+        if self.account_key.is_some() {
+            ds.field("account_key", &"<redacted>");
+        }
+
+        ds.finish_non_exhaustive()
+    }
 }
 
 impl Builder {
     pub fn root(&mut self, root: &str) -> &mut Self {
-        self.root = if root.is_empty() {
-            None
-        } else {
-            Some(root.to_string())
-        };
+        if !root.is_empty() {
+            self.root = Some(root.to_string())
+        }
 
         self
     }
+
     pub fn container(&mut self, container: &str) -> &mut Self {
         self.container = container.to_string();
 
         self
     }
     pub fn endpoint(&mut self, endpoint: &str) -> &mut Self {
-        self.endpoint = Some(endpoint.to_string());
+        if !endpoint.is_empty() {
+            self.endpoint = Some(endpoint.to_string());
+        }
 
         self
     }
-    pub fn credential(&mut self, credential: Credential) -> &mut Self {
-        self.credential = Some(credential);
+    pub fn account_name(&mut self, account_name: &str) -> &mut Self {
+        if !account_name.is_empty() {
+            self.account_name = Some(account_name.to_string());
+        }
 
         self
     }
+    pub fn account_key(&mut self, account_key: &str) -> &mut Self {
+        if !account_key.is_empty() {
+            self.account_key = Some(account_key.to_string());
+        }
+
+        self
+    }
+
     pub async fn finish(&mut self) -> Result<Arc<dyn Accessor>> {
         info!("backend build started: {:?}", &self);
 
@@ -113,11 +150,10 @@ impl Builder {
         // Handle endpoint, region and container name.
         let container = match self.container.is_empty() {
             false => Ok(&self.container),
-            true => Err(Error::Backend {
-                kind: Kind::BackendConfigurationInvalid,
-                context: HashMap::from([("container".to_string(), "".to_string())]),
-                source: anyhow!("container is empty"),
-            }),
+            true => Err(other(BackendError::new(
+                HashMap::from([("container".to_string(), "".to_string())]),
+                anyhow!("container is empty"),
+            ))),
         }?;
         debug!("backend use container {}", &container);
 
@@ -126,44 +162,22 @@ impl Builder {
             None => "blob.core.windows.net".to_string(),
         };
 
-        let mut context: HashMap<String, String> = HashMap::from([
-            ("endpoint".to_string(), endpoint.to_string()),
+        let context = HashMap::from([
             ("container".to_string(), container.to_string()),
+            ("endpoint".to_string(), endpoint.to_string()),
         ]);
 
-        let mut account_name = String::new();
-        let mut account_key = String::new();
-        if let Some(cred) = &self.credential {
-            context.insert("credential".to_string(), "*".to_string());
-            match cred {
-                Credential::HMAC {
-                    access_key_id,
-                    secret_access_key,
-                } => {
-                    account_name = access_key_id.to_string();
-                    account_key = secret_access_key.to_string();
-                }
-                // We don't need to do anything if user tries to read credential from env.
-                Credential::Plain => {
-                    warn!("backend got empty credential, fallback to read from env.")
-                }
-                _ => {
-                    return Err(Error::Backend {
-                        kind: Kind::BackendConfigurationInvalid,
-                        context: context.clone(),
-                        source: anyhow!("credential is invalid"),
-                    });
-                }
-            }
-        }
         let client = hyper::Client::builder().build(hyper_tls::HttpsConnector::new());
 
         let mut signer_builder = Signer::builder();
-        signer_builder
-            .account_name(&account_name)
-            .account_key(&account_key);
+        if let (Some(name), Some(key)) = (&self.account_name, &self.account_key) {
+            signer_builder.account_name(name).account_key(key);
+        }
 
-        let signer = signer_builder.build().await?;
+        let signer = signer_builder
+            .build()
+            .await
+            .map_err(|e| other(BackendError::new(context, e)))?;
 
         info!("backend build finished: {:?}", &self);
         Ok(Arc::new(Backend {
@@ -172,7 +186,7 @@ impl Builder {
             signer: Arc::new(signer),
             container: self.container.clone(),
             client,
-            account_name,
+            account_name: mem::take(&mut self.account_name).unwrap_or_default(),
         }))
     }
 }
@@ -214,7 +228,7 @@ impl Backend {
             .trim_start_matches('/')
             .to_string()
     }
-    #[allow(dead_code)]
+
     pub(crate) fn get_rel_path(&self, path: &str) -> String {
         let path = format!("/{}", path);
 
@@ -230,7 +244,7 @@ impl Backend {
 #[async_trait]
 impl Accessor for Backend {
     #[trace("read")]
-    async fn read(&self, args: &OpRead) -> Result<BoxedAsyncReader> {
+    async fn read(&self, args: &OpRead) -> Result<BytesReader> {
         increment_counter!("opendal_azure_read_requests");
 
         let p = self.get_abs_path(&args.path);
@@ -247,26 +261,39 @@ impl Accessor for Backend {
                     &p, args.offset, args.size
                 );
 
-                Ok(Box::new(ByteStream(resp).into_async_read()))
+                Ok(Box::new(
+                    resp.into_body()
+                        .into_stream()
+                        .map_err(move |e| other(ObjectError::new("read", &p, e)))
+                        .into_async_read(),
+                ))
             }
-            _ => Err(parse_error_response(resp, "read", &p).await),
+            _ => Err(parse_error_response_with_body(resp, "read", &p).await),
         }
     }
+
     #[trace("write")]
-    async fn write(&self, r: BoxedAsyncReader, args: &OpWrite) -> Result<usize> {
+    async fn write(&self, args: &OpWrite) -> Result<BytesWriter> {
         let p = self.get_abs_path(&args.path);
         debug!("object {} write start: size {}", &p, args.size);
 
-        let resp = self.put_blob(&p, r, args.size).await?;
+        let (tx, body) = new_http_channel();
 
-        match resp.status() {
-            http::StatusCode::CREATED | http::StatusCode::OK => {
-                debug!("object {} write finished: size {:?}", &p, args.size);
-                Ok(args.size as usize)
+        let req = self.put_blob(&p, args.size, body).await;
+
+        let bs = HttpBodyWriter::new(args, tx, self.client.request(req), |op, resp| {
+            match resp.status() {
+                http::StatusCode::CREATED | http::StatusCode::OK => {
+                    debug!("object {} write finished: size {:?}", &op.path, op.size);
+                    Ok(())
+                }
+                _ => Err(parse_error_response_without_body(resp, "write", &op.path)),
             }
-            _ => Err(parse_error_response(resp, "write", &p).await),
-        }
+        });
+
+        Ok(Box::new(bs))
     }
+
     #[trace("stat")]
     async fn stat(&self, args: &OpStat) -> Result<Metadata> {
         increment_counter!("opendal_azure_stat_requests");
@@ -336,9 +363,10 @@ impl Accessor for Backend {
                 debug!("object {} stat finished", &p);
                 Ok(m)
             }
-            _ => Err(parse_error_response(resp, "stat", &p).await),
+            _ => Err(parse_error_response_with_body(resp, "stat", &p).await),
         }
     }
+
     #[trace("delete")]
     async fn delete(&self, args: &OpDelete) -> Result<()> {
         increment_counter!("opendal_azure_delete_requests");
@@ -348,13 +376,27 @@ impl Accessor for Backend {
 
         let resp = self.delete_blob(&p).await?;
         match resp.status() {
-            StatusCode::NO_CONTENT => {
+            StatusCode::NO_CONTENT | StatusCode::ACCEPTED => {
                 debug!("object {} delete finished", &p);
                 Ok(())
             }
-            _ => Err(parse_error_response(resp, "delete", &p).await),
+            _ => Err(parse_error_response_with_body(resp, "delete", &p).await),
         }
     }
+
+    #[trace("list")]
+    async fn list(&self, args: &OpList) -> Result<ObjectStreamer> {
+        increment_counter!("opendal_azblob_list_requests");
+
+        let mut path = self.get_abs_path(&args.path);
+        // Make sure list path is endswith '/'
+        if !path.ends_with('/') && !path.is_empty() {
+            path.push('/')
+        }
+        debug!("object {} list start", &path);
+
+        Ok(Box::new(AzblobObjectStream::new(self.clone(), path)))
+    }
 }
 
 impl Backend {
@@ -373,7 +415,7 @@ impl Backend {
         if offset.is_some() || size.is_some() {
             req = req.header(
                 http::header::RANGE,
-                HeaderRange::new(offset, size).to_string(),
+                BytesRange::new(offset, size).to_string(),
             );
         }
 
@@ -385,21 +427,17 @@ impl Backend {
 
         self.client.request(req).await.map_err(|e| {
             error!("object {} get_object: {:?}", path, e);
-            Error::Object {
-                kind: Kind::Unexpected,
-                op: "read",
-                path: path.to_string(),
-                source: anyhow::Error::from(e),
-            }
+            other(ObjectError::new("read", path, e))
         })
     }
+
     #[trace("put_blob")]
     pub(crate) async fn put_blob(
         &self,
         path: &str,
-        r: BoxedAsyncReader,
         size: u64,
-    ) -> Result<hyper::Response<hyper::Body>> {
+        body: Body,
+    ) -> hyper::Request<hyper::Body> {
         let mut req = hyper::Request::put(&format!(
             "https://{}.{}/{}/{}",
             self.account_name, self.endpoint, self.container, path
@@ -407,24 +445,14 @@ impl Backend {
 
         req = req.header(http::header::CONTENT_LENGTH, size.to_string());
 
-        req = req.header(HeaderName::from_static(BLOB_TYPE), "BlockBlob");
+        req = req.header(HeaderName::from_static(X_MS_BLOB_TYPE), "BlockBlob");
 
         // Set body
-        let mut req = req
-            .body(hyper::body::Body::wrap_stream(ReaderStream::new(r)))
-            .expect("must be valid request");
+        let mut req = req.body(body).expect("must be valid request");
 
         self.signer.sign(&mut req).await.expect("sign must success");
 
-        self.client.request(req).await.map_err(|e| {
-            error!("object {} put_object: {:?}", path, e);
-            Error::Object {
-                kind: Kind::Unexpected,
-                op: "write",
-                path: path.to_string(),
-                source: anyhow::Error::from(e),
-            }
-        })
+        req
     }
 
     #[trace("get_blob_properties")]
@@ -444,12 +472,7 @@ impl Backend {
 
         self.client.request(req).await.map_err(|e| {
             error!("object {} head_object: {:?}", path, e);
-            Error::Object {
-                kind: Kind::Unexpected,
-                op: "stat",
-                path: path.to_string(),
-                source: anyhow::Error::from(e),
-            }
+            other(ObjectError::new("stat", path, e))
         })
     }
 
@@ -468,34 +491,69 @@ impl Backend {
 
         self.client.request(req).await.map_err(|e| {
             error!("object {} delete_object: {:?}", path, e);
-            Error::Object {
-                kind: Kind::Unexpected,
-                op: "delete",
-                path: path.to_string(),
-                source: anyhow::Error::from(e),
-            }
+            other(ObjectError::new("delete", path, e))
+        })
+    }
+
+    #[trace("list_blobs")]
+    pub(crate) async fn list_blobs(
+        &self,
+        path: &str,
+        next_marker: &str,
+    ) -> Result<hyper::Response<hyper::Body>> {
+        let mut uri = format!(
+            "https://{}.{}/{}?restype=container&comp=list&delimiter=/",
+            self.account_name, self.endpoint, self.container
+        );
+        if !path.is_empty() {
+            uri.push_str(&format!("&prefix={}", path))
+        }
+        if !next_marker.is_empty() {
+            uri.push_str(&format!("&marker={}", next_marker))
+        }
+
+        let mut req = hyper::Request::get(uri)
+            .body(hyper::Body::empty())
+            .expect("must be valid request");
+
+        self.signer.sign(&mut req).await.expect("sign must success");
+
+        self.client.request(req).await.map_err(|e| {
+            error!("object {} list_object: {:?}", path, e);
+            other(ObjectError::new("list", path, e))
         })
     }
 }
-struct ByteStream(hyper::Response<hyper::Body>);
 
-impl futures::Stream for ByteStream {
-    type Item = std::io::Result<bytes::Bytes>;
+fn parse_error_response_without_body(
+    resp: Response<Body>,
+    op: &'static str,
+    path: &str,
+) -> io::Error {
+    let (part, _) = resp.into_parts();
+    let kind = match part.status {
+        StatusCode::NOT_FOUND => ErrorKind::NotFound,
+        StatusCode::FORBIDDEN => ErrorKind::PermissionDenied,
+        _ => ErrorKind::Other,
+    };
 
-    fn poll_next(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
-        Pin::new(self.0.body_mut())
-            .poll_next(cx)
-            .map_err(|e| std::io::Error::new(std::io::ErrorKind::Other, e.to_string()))
-    }
+    io::Error::new(
+        kind,
+        ObjectError::new(op, path, anyhow!("response part: {:?}", part)),
+    )
 }
 
 // Read and decode whole error response.
-async fn parse_error_response(resp: Response<Body>, op: &'static str, path: &str) -> Error {
+async fn parse_error_response_with_body(
+    resp: Response<Body>,
+    op: &'static str,
+    path: &str,
+) -> Error {
     let (part, mut body) = resp.into_parts();
     let kind = match part.status {
-        StatusCode::NOT_FOUND => Kind::ObjectNotExist,
-        StatusCode::FORBIDDEN => Kind::ObjectPermissionDenied,
-        _ => Kind::Unexpected,
+        StatusCode::NOT_FOUND => ErrorKind::NotFound,
+        StatusCode::FORBIDDEN => ErrorKind::PermissionDenied,
+        _ => ErrorKind::Other,
     };
 
     // Only read 4KiB from the response to avoid broken services.
@@ -511,18 +569,20 @@ async fn parse_error_response(resp: Response<Body>, op: &'static str, path: &str
                     break;
                 }
             }
-            Err(e) => return Error::Unexpected(anyhow!("parse error response parse: {:?}", e)),
+            Err(e) => return other(anyhow!("parse error response parse: {:?}", e)),
         }
     }
 
-    Error::Object {
+    io::Error::new(
         kind,
-        op,
-        path: path.to_string(),
-        source: anyhow!(
-            "response part: {:?}, body: {:?}",
-            part,
-            String::from_utf8_lossy(&bs)
+        ObjectError::new(
+            op,
+            path,
+            anyhow!(
+                "response part: {:?}, body: {:?}",
+                part,
+                String::from_utf8_lossy(&bs)
+            ),
         ),
-    }
+    )
 }
